@@ -1,15 +1,24 @@
-# this is stuff that needs to be run first and in order for S4 crap to work
-methods::setOldClass("telemetry")
-new.telemetry <- methods::setClass("telemetry", representation(info="list"), contains="data.frame")
+# this is stuff that needs to be run first (and in the right order) for S4 crap to work
 
-methods::setOldClass("UD")
-new.UD <- methods::setClass("UD", representation(info="list"), contains="list")
+methods::setOldClass("UERE")
+new.UERE <- methods::setClass("UERE",contains="matrix",representation=methods::representation(DOF="matrix",AICc="numeric"),
+                              prototype=methods::prototype(matrix(),DOF=matrix(),AICc=numeric()))
+
+methods::setOldClass("telemetry")
+new.telemetry <- methods::setClass("telemetry",contains="data.frame",representation=methods::representation(info="list",UERE="UERE"),
+                                   prototype=methods::prototype(data.frame(),info=list(),UERE=new.UERE()) )
 
 methods::setOldClass("ctmm")
-new.ctmm <- methods::setClass("ctmm", representation(info="list"), contains="list")
+new.ctmm <- methods::setClass("ctmm",contains="list",representation=methods::representation(info="list"),
+                              prototype=methods::prototype(list(),info=list()))
+
+methods::setOldClass("UD")
+new.UD <- methods::setClass("UD",contains="list",representation=methods::representation(info="list",type="character",CTMM="ctmm"),
+                            prototype=methods::prototype(list(),info=list(),type=character(),CTMM=new.ctmm()))
 
 methods::setOldClass("variogram")
-new.variogram <- methods::setClass("variogram",representation("data.frame",info="list"))
+new.variogram <- methods::setClass("variogram",representation=methods::representation("data.frame",info="list"),
+                                   prototype=methods::prototype(data.frame(),info=list()))
 
 # existing functions -> S4 generics
 # this doesn't work
